/* wd.c: A WD80x3 ethernet driver for linux. */
/*
	Written 1993-94 by Donald Becker.

	Copyright 1993 United States Government as represented by the
	Director, National Security Agency.

	This software may be used and distributed according to the terms
	of the GNU Public License, incorporated herein by reference.

	The author may be reached as becker@CESDIS.gsfc.nasa.gov, or C/O
	Center of Excellence in Space Data and Information Sciences
	   Code 930.5, Goddard Space Flight Center, Greenbelt MD 20771

	This is a driver for WD8003 and WD8013 "compatible" ethercards.

	Thanks to Russ Nelson (nelson@crnwyr.com) for loaning me a WD8013.

	Changelog:

	Paul Gortmaker	: multiple card support for module users, support
			  for non-standard memory sizes.
			 

*/

static const char *version =
	"wd.c:v1.10 9/23/94 Donald Becker (becker@cesdis.gsfc.nasa.gov)\n";

#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <asm/io.h>
#include <asm/system.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include "8390.h"

/* A zero-terminated list of I/O addresses to be probed. */
static unsigned int wd_portlist[] =
{0x300, 0x280, 0x380, 0x240, 0};

int wd_probe(struct device *dev);
int wd_probe1(struct device *dev, int ioaddr);

static int wd_open(struct device *dev);
static void wd_reset_8390(struct device *dev);
static void wd_get_8390_hdr(struct device *dev, struct e8390_pkt_hdr *hdr,
						int ring_page);
static void wd_block_input(struct device *dev, int count,
						  struct sk_buff *skb, int ring_offset);
static void wd_block_output(struct device *dev, int count,
							const unsigned char *buf, const start_page);
static int wd_close_card(struct device *dev);


#define WD_START_PG		0x00	/* First page of TX buffer */
#define WD03_STOP_PG	0x20	/* Last page +1 of RX ring */
#define WD13_STOP_PG	0x40	/* Last page +1 of RX ring */

#define WD_CMDREG		0		/* Offset to ASIC command register. */
#define	 WD_RESET		0x80	/* Board reset, in WD_CMDREG. */
#define	 WD_MEMENB		0x40	/* Enable the shared memory. */
#define WD_CMDREG5		5		/* Offset to 16-bit-only ASIC register 5. */
#define	 ISA16			0x80	/* Enable 16 bit access from the ISA bus. */
#define	 NIC16			0x40	/* Enable 16 bit access from the 8390. */
#define WD_NIC_OFFSET	16		/* Offset to the 8390 from the base_addr. */
#define WD_IO_EXTENT	32


/*	Probe for the WD8003 and WD8013.  These cards have the station
	address PROM at I/O ports <base>+8 to <base>+13, with a checksum
	following. A Soundblaster can have the same checksum as an WDethercard,
	so we have an extra exclusionary check for it.

	The wd_probe1() routine initializes the card and fills the
	station address field. */

#ifdef HAVE_DEVLIST
struct netdev_entry wd_drv =
{"wd", wd_probe1, WD_IO_EXTENT, wd_portlist};
#else

int wd_probe(struct device *dev)
{
	int i;
	int base_addr = dev ? dev->base_addr : 0;

	if (base_addr > 0x1ff)		/* Check a single specified location. */
		return wd_probe1(dev, base_addr);
	else if (base_addr != 0)	/* Don't probe at all. */
		return ENXIO;

	for (i = 0; wd_portlist[i]; i++) {
		int ioaddr = wd_portlist[i];
		if (check_region(ioaddr, WD_IO_EXTENT))
			continue;
		if (wd_probe1(dev, ioaddr) == 0)
			return 0;
	}

	return ENODEV;
}
#endif

int wd_probe1(struct device *dev, int ioaddr)
{
	int i;
	int checksum = 0;
	int ancient = 0;			/* An old card without config registers. */
	int word16 = 0;				/* 0 = 8 bit, 1 = 16 bit */
	const char *model_name;
	static unsigned version_printed        _addr);RS0; i < 6;i;
	int c+trl_reg = inb(i8dr, Pur_addrl_reg = inb(i8_READ(0)) 6			0ave asionarhe d
statiing.he ethercv) ==l_reg = inb(i9_READ(0))rq] != i;
	int c length & 0x7FF) all. */
		return 
us);
		pt.  We

/* We"dev"the 83Godda.c 	Probe _name;
ort fort *	thes well. ;

	if (dev == NULL) {
(sion = Pwe area (dev unknown deinit(devctrl_it_ <linux/(set0)return 0;
 zero.  Chsemi-vne he shv->tra/dica/* Fla recouseli be shared. _port = (dev->me%, 0x0t_rx != debug = dev->%, 0x0t_r == NULL) {
(gcc _WARNING sion =  modcouseli be shv->tra 	Pr = dev->ce me
	8kB   /* xclu-ned longn deinit(devt = (dev->meince_rxdebug = dev->=		return ed. *i	if (neIG_Iunsigned version    tbusy) != 0)
		printk(versL) {
			prinwd.c: Ar founx, :%02X.\n", devev, int iint i;
		for (i = 0; i < 6;ata;
			p%2.2X"PAR0 + i, dev->dev trl_reg = inb(i8dr, P_CONTROL);

/* This foPuren Spa* Don'llowitpacit slnd distbyScienMike Jagcke <jaggy@pur	: t.dshann@c.uk>.oPurerintkdethi

	This Scien/* SLIP confiand vet is curansmi<linhi

ard,
	so he  should chehasm
ls;
					detectecksu	/* 0 = t, 1 =taboumb (Toshibaata mpeong * An m
ls;
shared. *_reg = in+0_READ'P'IG_I*_reg = in+_mode 'D' == NU		const unsign co5 trl_reg = in+5>if_poQ line, *_reg = in+2r == NUter, ART3: */
	int wor t char *mot w"PDI8023-8"; = 0;
			bter, ART5: */
	int wor t char *mot w"PDUC8023"; = 0;
			bter, ARTa: */
	int w1r t char *mot w"PDI8023-16"; );
					br			0t.  Inx77)= dumb) 	Probeyhinkrelea area lloc	printkthercv)d.  Its:	 */
	int wor t char *mot w"PDI8023"; = 0;
			b}it(devt = (dev->meinelseo5 ONTROc)dr,0xc0_R<< 12;7;
	else
		devlseo5 ONTRe0_READ(0eL) &10aselseo5 >t_deb+		retuturn;
	4x.\n",
			0xcatesPuren Spa* Don'ercv)CHECKER
 acces	:	A l line c* Checfset tlize t1

	e slip (borrowd filterrup.c is is d WA Chm	:  transfe1

	a porhe  "tad reit-only . ART3RegisteI'muff;	ar-time usiwotransfe1

	slphaeepali we onyicertachangeyegisteupt and n have ttioIts.'ercv)CHECshould cheha			/*	/* 0lize t1-ter hernme ASIC r0/8	sliaChecRegisteDos.
   shouli>=6rpora --ter hech
   ha		/*	 theize ts!'ercv)nt i;
		for (i = 0; i < 6; ed. *_reg = in+ PAR0 *_reg = in+8+ Pebug >= 0;
			b			els>=_debug > 0;
	int an_busy	t char *mot w"for th-		/"ev->pr/
	int wor
Xmit);
			} eld. */mp trl_reg = in+1ISR_Rxfid:
  se>+116* 0liiRedundanne. **/mp ^nx77)evev, inst ISR_Rxa				/* Att	Set o16* 0liiRedundaned. _ *_re g = in+_mo&0x77) != 0x77) !390.h t, 1 = An oansmit.G_I(/mp &0x77) != 0x77)	)
	4x.\ET_BHhecfst ouetransmit. Goveric_ co5 trl_reg = in+#define WDd_status);M int Att		/*it-onound upy a wormal Rx mode.ne. **defin != eric_ co5&_add)evev, ins#define WDd_statune. */mpevev, instd_statut char *mot w"for 13"o ReTx"/
	int w1rtus);
		

/* We16* 0lize t1pora! layers.t);
			} elst char *mot w"for th"o ReTx"/
	int wce_rx }
atune. */mpevev, instd_e = 0; 	iled, ave the o co1a/* Fl Rx mod}org>
 */
fthe _	printk		b			e !0;
	int G_I(*_reg = in+_mo&0x77) !R0 ("/
	int&0x77) )b->data;
			\nfor ?3: Bus  *  tr access ,Rx a(* Don !R0 x a(ed_credev->02x%02x..","/
	int?cfst:(ioad*_reg = in+_mo&0x77) !?cfst:(itp_timer);
#enfig.h>

#if #defHMEMFIG_I#defHMEMase_ar t00NTROLA Thi13, s:
 *  
   s can bee.	er mode. *= (dev->mein#defHMEMeader[4];7 : 4;
	if (dev->meie & 1) == 0)Sanal She c		/*	 the shoulbuffer. *ed_0 trl_reg = in)ut <<= 1;
_0 tAD(0))  == 0_0 tAD(ebug > G		0uTY s plan:lter fc.  We shoula flocobe all likely l
ftrsinterrupt.evt = (dev->meincxd t00kb->data;
			pwe onsne chtation %#lx"PAR0 + = (dev->m)r
Xmit);
			} eld. *ite_dev->	int numl_reg = in+#define WDd ONTROf			br			/
	
	ize tsckkk!  I donProces ASIC re --ter upt ansD(0)).edundaned. ite_dev->	int nu 0x78)  =="/
	int oaddr) =	ite_dev->	int numx77);rupt.evt = (dev->meinelseo0&_a3f_R<< 13eb+	 ite_dev->	int n<< 19data[13]}rx_head;
#endTset to t devicon on wlse chtation --t	/* Ft-only s/* Thtpora! layeULL);

	dev->base_addr +#define WD_NIase_addr;

	i
		d<_debug >terruptirq2, por9,3,5,7,10,11,15,4};ffer. *ed_1 trl_reg = in+1ISffer. *ed_4 trl_reg = in+4)ut <<= 10;
	int  == 0_1 tAD(0)))
	4rs++;
!!;   -reboot wa
	/* Find! layers>priv;nic_v->base_addr +#define WD_NIase_tus);
		

/* W;			/-stylepatible" efine bdethevicredev- forgindead.padthe IReDos/* O
		dootf	}
	/* Find the IRNo Termon onr fISs.
 failureskrelocal  -reboot {
				iggering an interrutune. _p(EstrucretMAb+	Estrucne W,;nic_v->bd_statne. */
	outnic_v->b+EN0_IMRh(NET_v;

	/* Dctiontper layers/* O
		/
	ethe0d_statne. _p(   outnic_v->ba+	EN0_IMRh(NET_0x40	/*Dctiont LPT interrupttne. _p(  	outnic_v->ba+	EN0_RCNTLOd_statne. _p(  	outnic_v->ba+	EN0_RCNTHId_statne. *EstrucREE_D+EstrucneART,;nic_v->bd_endif{
				iit..nterrupt.evt 
		dev/* O
		/redev-(2d_statne. _p(  	outnic_v->b+EN0_IMRh(NET_terruDctiontper  for i neede	 ed. *i	if (net_debug > 2)
				s/* O
		dr f%d"PAR0 + ce.\n", _addr;

	i
		d<_deusy = 0;
	
		dev"/
	int?cf0ase5			break;
		}  0;
	
		devuptirq2elseo4 >t_deb&0x773eb+	  0_1 &0x774)]gth > 0);
	} el0;
	
		de	if lse {
	xup staosal :Find2if_port" isind9hercv)d.;
	
		dev9CONTROLSnarpt, so if the itine fen.
		' ther     1

	waintk("	num_ard,cannointerrble taddress,lize t1ween s PROM, b	/* .. wbe shared.  0
		|| request_irq(de*i	q, &net_interrt char *mo0, "ATP", NULLL) {
		p ux40	/* ocket( %#3lxg packet.\nce.\n", irq);
 return -EAGAIROLA Th	/* re is no dehe card 1

	 to t singlece_addVarious fie ed. *th, voirdware_i)
	4NULLL) {
		p ux40	/* ocket(  /* No meme is no dn deinit(TROL);
	free_irq(dev->irq,  NULL)
		return -EAGAIROLOK,ardr /* Thcertachang771

	god accor the feS	eth is slap ths fie  0
		||  (check_region(ioaddr, WD_Irrt char *mo>if_p*i	head.r. *mot wt char *model*i	head.r."/
	int w"/
	indel*i	head.r.tx_, const stein#define WD_del*i	head.r.rx_, const stein#define WD_ jifX_PAGESev->privkk!  irqshifting le the shareo be less,lize t1

	TctPROM,  intwbe sharree_ir= (dev->mein.evt = (dev->mejifX_PAGES*256CONTROLS
	
	  Thes(d_cfor thEBT)	   Thisa mpeong supporxes 32k!)e shared memx7 : 4;
	if (dev->!AD(ebug >*i	head.r.;
	dst stein 4;
	if (dev->-AR0 + = (dev->m)/256COmit);
			} e*i	head.r.;
	dst stein"/
	int?c#define WD13 :
#define WD03e_rxdebug = dev->=	.evt = (dev->meji(*i	head.r.;
	dst ste-n#define WD_)*256COmiarree_ir= (dev->=	.evt = (dev-versL) {
			 %sat %#3lx,  le the shareor foulx-oulxror %04xerrt char *mo0,r,
		   dev->ir= (dev->m,	.evt = (dev--1>if_p*i	head.r.void wd_reast_ic void wd_redel*i	head.r.void wd_bloast_ic void wd_blodel*i	head.r.void wing thest_ic void wing thdel*i	head.r.void wd_get_est_ic void wd_get_;nd & 7;

	dest_ic 		= net_open;
	dest_ic  int wd_clnetNS wd_girdware_ddr, IMfig.1IRQOUT);
	/* if the itgntwrdocumenta

	Twithout  Thes--tM.U memxTROL);tbuspoe obM, po Tnt's espunsafte-n9/23/94shared. *_reg = in+14eb&0x72ddr) ne. *l_reg = in+4)|_ar evev, ins4) rx_head;
#SET);

	return(). */
static int wd_open(struct deviignored)
{	short ioaddr = dev->be-n#define WD_NIaped; define Wueue     *Mrqshifting le the share.LA -rest		/*me ASIC r0 A Sog enougae isnWD8013 "co se>+1re (a		/*ize tsatistic*i	head.r.vo_0 tr _port = (dev->m>>12_h) >>3f_R|#define	 W;tic*i	head.r.vo_5 tr _port = (dev->m>>19d ONTROf_R|#definase_addr;*i	head.r."/
	inack dne. **i	head.r.vo_5evev, ins#define WDd_s dne. **i	head.r.vo_0evev, int ped; define Wueue   *i	 int device  MODifde_USE_COUNTce  SET);

	return(). */
static void wd_reset_8390(struct devi ioaddr)w	if (rdev->ifoaddr = dev->be-n#define WD_NIaped; define Wueue TROL);
efine	 ,)w	if (rdev-Pur_addr*i	if (net_1ebug > 3) void t-time uswd.c: At=%lu.04x %pires =)del*i	head.r.tx-timv->irq] = 

/*th is sit-only ASIC s,	a pori fun
		over if soto spodhehasmRQ line. *( _port = (dev->m>>12_h) >>3f_|define	 W),)w	if (rdev-Pur_addr*i	head.r."/
	inackine. *defin != _port = (dev->m>>19d ONTROf_,)w	if (rdev-s#define WDd_sr_addr*i	if (net_1ebug > 3) void s++;
 deinittPkt);
	retn 0;rabffset to t singlece	/* fo. Similar* Offsetvoid wd_bloarobe1()->tbu
erruptkkk!  d" only neec be ne isse>+1r-timwrrqsane rese/* forween on't ing that->tra 	fo remg get ruptoptimizetributed aled memrn(). */
static void wd_get_8390_hdr(struct device *dev, struct e8390_pkt_h ,
						int rita_moddr)w	if (ed_cifoaddr = dev->be-n#define WD_NIaped; define Wueue	   */
  unsignkt_dev->mein.evt = (dev->mejiels			int re-n#define WD_)<<8)n 
us);
	'en a -restket(a 4 to neruct rx_wa
	/* Thinterrup outfillrrl_LNsonterruptex40	/* Enablevaluebefoblem whruct r,h the Tx and ? No Rss,lizded memx7 : *i	head.r."/
	inackine. *defin !=*i	head.r.vo_5evw	if (ed_s#define WDd_sre_addr;
#ifdef SET	16it's espng771

	tion upt
	  do evertbusProcesadl()1

	f Soundmemx= (cpy_orroio(kt_h kt_dev->m,d memof(e *dev, struct e8390))eader[4];((
static unsi*)390)[0]    *p+l(kt_dev->m) rx_head;retn 0	}

	/d_bloahe c	ng the
	  ev;		nta
le the shareoatible" et,h theto d's ing oiftingWes5302
digital* An olroblem w: if therchoruct	fohowwitchoheck mean!
	 16-bWD80liikely l
* Thtpon on wr-time of TXwrrqt,h thed" only irq kern line, of sec 8-h thefset tlvalusd memrn(). */
static void wd_block_input(struct device *dev, in
						  struct sk_buff *skb, int ring ioaddr)w	if (ed_cifoaddr = dev->be-n#define WD_NIaped; define Wueue	   */
  unsignxfe_dev->mein.evt = (dev->mejikb, int rine-n(#define WD_<<8)n 
u7 : xfe_dev->me+ (--bogusree_ir= (dev- 1) == 0) /*
	 *	wrrqs, so i the hadw buffer. *semi_s, bogusree_ir= (dev->-nxfe_dev->me_rx= (cpy_orroio(*data = s,nxfe_dev->m,*semi_s, bogs(fla, bog-=*semi_s, boe_rx= (cpy_orroio(*data = se+ semi_s, bo, ree_ir= (dev->m, s, bogs(feturn;
	} elsePutfillif chao	Makeunks--trd,can s,pye+ (int w bufferotoio_s,py_ th_nt ree_skxfe_dev->m,*s, bo, 0)return 0;
T);

 tur Enable 16 bitt boot,  is non-_bit.	from trae -damt rememx7 : *i	head.r."/
	inackine. **i	head.r.vo_5evw	if (ed_s#define WDd_s}mrn(). */
static void wd_block_output(struct device *dev, in 
							const unsigned chsmit. Gov, const sta ioaddr)w	if (ed_cifoaddr = dev->be-n#define WD_NIaped; define Wueue	nsignsh= (ein.evt = (dev->mejiel, const ste-n#define WD_)<<8)n 
mx7 : *i	head.r."/
	ina
	} elseT);

 nahe c	tur Enable 16 bitt boot,  is non-_bit.e 0 or 1 */defin !=*i	head.r.vo_5evw	if (ed_s#define WDd_srx= (cpy_toio(*h= (, lengts, bogs(flne. **i	head.r.vo_5evw	if (ed_s#define WDd_sbreak;
		}= (cpy_toio(*h= (, lengts, bogs(}
urn(). */
static int wd_close_card(struct devi ioaddr)w	if (ed_cifoaddr = dev->be-n#define WD_NIaped; define Wueuer_addr*i	if (net_1ey) != 0)
		prinShu t-timo troatible" econflict.\n", dev->ni int
nedevice INUSE	0	grupt lifset tlly 8et tl
le the shareoso  is non-_bit.e 0 o7 : *i	head.r."/
	inackine. **i	head.r.vo_5evw	if (ed_eji#define WD ice INUSAthe Tx and  Enable the shared memine. **i	head.r.vo_0gth define	 Wevw	if (ed_ice IMODiDEe_USE_COUNTce
#SET);

	returre_addr;MODULEaddr. */
MAX_#defARDS	4	;      	/* MAX nuwdt  Thesnforort fore_addr. */
NAMELEN		8outfill;	eceive  Chstor-timct.\n", d probed. */nsign", d wd_pNAMELEN *
MAX_#defARDS, porterr};osed. */
statiche SLIP ev_wd[MAX_#defARDS, porress. v->i, elsewe ons13, wunksl;	", d wd_p], oThi1 0 or0nterrerrer or0nter or0nterrerrv->i,rv->isbr,#endi_page);
staio[MAX_#defARDS, porterr};osed. */terrupt[MAX_#defARDS,  porterr};osed. */terr sh[MAX_#defARDS, porterr};osed. */terr= (dev-[MAX_#defARDS, porterr};*slcomp;nnta
td.r= (d mem 00- FvK20771

	s
/*th t boot,  16-b* Check a/* O* Don'takes plaLIPpundblll.
rom lap the * O* Dons
 nah run for lachoutpure>ce mreto Aev-wbe sha
stal_it_ort fo_bit(i ioaddr)ng77_devicf /* oprinted     ng77_dev		for ng77_dev	<
MAX_#defARDSr ng77_devst[i]; iegs)
{
	struct device& ev_wd[ng77_dev]e_rxdebug *mot w", d wd_+(NAMELEN*ng77_dev);7;
	else
		devupt[ng77_dev]e_rxdebug

	dev->base_a[ng77_dev]e_rxdebug= (dev->mein sh[ng77_dev]e_rxdebug= (dev->=	= (dev-[ng77_dev]e_rxdebugl_itt ioadd Don			b			ela[ng77_dev] tAD(ebx + 100) {ng77_dev	!AD(eb= 0;
	ets; 16-b** O* Don'1natenn'ercv)ULL) {
(gcc _NOTICE sion = Pt as r6-b** O* Dofor (ce mreto Aev-wb)c* ChecCheck ae" econfdata[13])<= 1;
_ASIC _
strucedevi>!AD(ebug >ULL) {
(gcc _WARNING sion =    -d.c: A An of /* o(i/onumx7%x)n devica[ng77_dev]\n", _addrf /* o!AD(ebSET);

	r*slcGoicon lea atennnterruptNULL)
		reeturna[13])f /* _packet
#SET);

	retur
stat	Setnup_ort fo_bit(i ioaddr)ng77_devnted     ng77_dev		for ng77_dev	<
MAX_#defARDSr ng77_devst[i]; iegs)
{
	struct device& ev_wd[ng77_dev]e_rx7 : 4;
	ino de!f (skb == NULed)
{	short ioaddr = dev->be-n#define WD_NIa NUL}

	d 4;
	ino d);rupt.evt no def (skb;ruptTROL);
	free_irq(dev->irq,   NULL);
	irq2dev_map[dev-(skb;ruptrelea af (check_region(ioaddr, WD_IO;g > 6);
_ASIC _
strucedeviata[13]}r}rx_head   *MODULEterr
de);
}

/*
 * Local variables:
 *  compile-command: "gcc -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -ion -c atp.c"
 *  version-control: 5
 *  tab-widtl: t
 *  kept-new-versionh: 4
 * End:
 */
